function [LPCC, A ,E, K, G, AC] = extract_LPCC(s, fe, win, pas, nac, nlpcc, plot_on);
% Extract the LPCC - Linear Predictive Cepstral Coeficients. The parameters
% of the function are the following:
% s, fe - input signal and his sampling frequency
% win - dimensions in samples of the analysis window 
% pas - the distance/step between two consecutive windows
% nac - the order of autocorrelation, which will be also the order of LPC 
% nlpcc - number of cepstral coefficients  (conversion from LPC coef.)
% plot_on = 'all' - plot all the graphics for all analysis windows
%         = '3D'  - plot only the 3D graphics
%         = numerical value which indicates a specific analysis window (implicit=nwin/2)
%         = 0     - no graphic will be ploted
% Return: -LPCC cepstral coefficients   -A (LPC coefficients)
%         -E prediction error           -K reflexion coefficients (PARCOR)
%         -G (Log Area Ratio Coef.)     -AC Autocorrelation coefficients
%--------------------------------------------------------------------------
% Copyright(c) Zbancioc Marius-Dan, 2014, zmarius@etti.tuiasi.ro
% 1)Institute of Computer Science, Roumanian Academy, Iasi Branch, 
% 2)Technical University "Gheorghe Asachi" of Iasi, Romania
%--------------------------------------------------------------------------
% References:
%- M. Zbancioc, M. Feraru, The Analysis of the FCM and WKNN Algorithms Performance for the Emotional Corpus SROL, Advances in Electrical and Computer Engineering, Vol. 12, Issue 3, 2012, pp. 33  38, ISSN: 1582-7445, e-ISSN: 1844-7600, Digital Object Identifier: 10.4316/AECE.2012.03005
%- M. Feraru, M. Zbancioc, Speech Emotion Recognition for SROL Database using Weighted KNN algorithm, International Conference on Electronics, Computers and Artificial Intelligence (ECAI2013), June 27-29, 2013, Pitesti, Romania, ISBN: 978-1-4673-4935-2, Digital Object Identifier: 10.1109/ECAI.2013.6636189 2013 IEEE
%- M. Zbancioc, M. Feraru, A Study about MFCC Relevance in Emotion Classification for SROL Database, Proceedings of 4th International Symposium on Electrical and Electronics Engineering, ISEEE 2013, October, 11-13, 2013, Gala?i, Romnia, ISBN 978-1-4799-2441-7, IEEE Catalog Number CFP1393K-USB.
%- M. Zbancioc, M. Feraru, Using the Lyapunov Exponent from Cepstral Coefficients for Automatic Emotion Recognition, Proc of 8th IEEE International Conference on Electrical and Power Engineering  EPE 2014, 16-17 Oct., Ia?i, Romnia, ISSN: 978-1-4799-5848-1.
%- M. Feraru, M. Zbancioc, Emotion Recognition in Romanian Language using LPC Features, The 4th IEEE International Conference on E-Health and Bioengineering - EHB 2013, November 21-23, 2013, Ia?i, Romania, ISBN: 978-1-4799-2373-1/13/$31.00 2013 IEEE

if nargin<2
    [fname,fpath]=uigetfile('*.wav','Open');
    %if ~strcmp(dir_cur,fpath(1:end-1)), cd(fpath); end
    [s,fe]=wavread([fpath fname]);
    dir_cur=pwd;
end
if nargin<5 | ~length(nac) | ~isnumeric(nac); nac=14; end
if nargin<6 | ~length(nlpcc) | ~isnumeric(nlpcc) 
    %nlpcc=nac; 
    nlpcc=fix(nac*3/2); 
end
%if nargin<7; plot_on='all'; end

% The analysis windows have 20ms (320 samples for a sampling freq. fe=16KHz)
% and a step of 10 ms (overlapping of 50%). It computes the autocorrelation
% of 14 orders and the LPC coef. using Levinson-Durbin method.
if nargin<3, win = round(20/1000*fe); end
if nargin<4, pas = round(10/1000*fe); end

len = length(s);
t = (0:pas:len-win)*win/fe; nwin=length(t);

if nargin<7; plot_on=0; end %plot_on=fix(nwin/2); end

AC = zeros(nwin,nac+1);

contor=1;
for i = 1 : pas : len-win+1
    %swin = s(i:i+win-1).*(0.54-0.46*cos(2*pi*[0:win-1]/(win-1))'); %method 1
    swin = s(i:i+win-1).*hamming(win);                              %method 2
    twin = [t(contor) : 1/fe : t(contor)+(win-1)/fe];

    if strcmp(lower(plot_on),'all') | plot_on==contor
        figure(1); 
        subplot(221); plot(twin,s(i:i+win-1)); xlabel('Temporal signal'); grid;
        ax=axis; ax=max(abs([ax(3) ax(4)])); axis([twin(1) twin(win) -ax ax]);
        subplot(222); plot(twin,swin); xlabel('Hamming windowed signal'); grid;
        axis([twin(1) twin(win) -ax ax]);        
    end

    % Each window is pre-emphasised using a HPF filter with the transfer
    % function H(z)=1-0.97z^-1
    swin=filter([1 -0.97],[1],swin);
    
    % Computes the autocorrelation of nac orders (implicit 14). If the
    % dimension of the window is less than nac (autocorrelation order) it
    % will complete the output signal with zeros.
    ac=zeros(2*nac+1,1);
    if nac>win
        if size(swin,1)~=1; swin=[swin; zeros(nac-win,1)]; end
        if size(swin,2)~=1; swin=[swin, zeros(1,nac-win)]; end
    end
    for j = 0 : nac
        % ar(j+1)=sum(swin(1:win-j).*conj(swin(j+1:win)));
        % se face automat si sumarea la inmultirea unei linii cu o coloana
        ac(j+1+nac)=sum(swin(1:win-j).*conj(swin(j+1:win)));
    end
    for j=1:nac; ac(j)=conj(ac(2*nac+2-j)); end
    %ac=xcorr(swin,nac);      % MATLAB function for autocorrelation
    AC(contor,:)=ac(nac+1:2*nac+1)'; contor=contor+1;

    if strcmp(lower(plot_on),'all') | plot_on==contor
        figure(1);
        subplot(223); plot(twin,swin); xlabel('Preemphasized Window'); grid;
        ax=axis; ax=max(abs([ax(3) ax(4)])); axis([twin(1) twin(win) -ax ax]);
        subplot(224); plot([1:nac+1],AC(contor-1,:)); xlabel([int2str(nac),'th Order Autocorrelation']); grid;
        if plot_on~=contor; pause; end
    end
end
%------------------------------------------------------------------------------
% Computes LPC coefficients from vectors obtained from autocorrelation
%------------------------------------------------------------------------------
A=zeros(nwin,nac+1);
E=zeros(nwin,1);
K=zeros(nwin,nac);
G=zeros(nwin,nac);

for k = 1 : nwin
    error=AC(k,1);
    a=zeros(nac,nac);
    for i = 1 : nac
        K(k,i)=AC(k,i+1);
        for j = 1 : i-1
            K(k,i)=K(k,i)-a(j,i-1)*AC(k,i-j+1);
        end
        K(k,i)=K(k,i)/error;
        a(i,i)=K(k,i);        
        for j = 1 : i-1
            a(j,i)=a(j,i-1)-K(k,i)*a(i-j,i-1);
        end
        error=(1-K(k,i)^2)*error;
        
        G(k,i)=log10((1+K(k,i))/(1-K(k,i)));
    end
    A(k,:)=[1 -a(:,nac)'];
    E(k)=error;
end
K=-K';

% The valus of the vectors are the same with those provided by MATLAB
% function LEVINSON (which can be used for LPC coef. comptuting with the
% Levinson-Durbin method)         [A,E,K]=levinson(AC');

%------------------------------------------------------------------------------
% Convert the LPC coefficients in cepstral coefficients. The order of LPCC
% can be higher than the autocorrelation order. In this case the modulo
% function is used because the vector size is given by the value of nac.
%------------------------------------------------------------------------------
LPCC=zeros(nwin,nlpcc);
method=1;

for k = 1 : nwin
    for i = 1 : nlpcc
        if method==1
            if i<= nac; LPCC(k,i)=A(k,i+1); end
            for j = 1 : i-1
                if mod(i-j,nac)==0
                    LPCC(k,i)=LPCC(k,i)+j/i*LPCC(k,j)*A(k,nac+1);
                else
                    LPCC(k,i)=LPCC(k,i)+j/i*LPCC(k,j)*A(k,mod(i-j,nac)+1);
                end
            end
        else
            LPCC(k,i) = A(k, mod(i,nac)+1+ (mod(i,nac)==0)*nac);
            for j = 1 : i-1
                if mod(j,nac)==0
                    LPCC(k,i)=LPCC(k,i)+(j-i)/i*LPCC(k,i-j)*A(k,nac+1);
                else
                    LPCC(k,i)=LPCC(k,i)+(j-i)/i*LPCC(k,i-j)*A(k,mod(j,nac)+1);
                end
            end
        end          
    end
end 

if isnumeric(plot_on) & plot_on>0 & plot_on<=nwin
    figure(2); 
    subplot(221); plot([1:nac],A(plot_on,2:nac+1)); ylabel('Time [msec]'); grid;
    xlabel([int2str(nac),'th Order LPC Coefficients']); 
    ax=axis; ax(1)=1; ax(2)=nac; axis(ax);
    subplot(222); plot([1:nac],K(:,plot_on)); ylabel('Time [msec]'); grid;
    xlabel([int2str(nac),'th PARCOR Coefficients']);
    ax=axis; ax(1)=1; ax(2)=nac; axis(ax);
    subplot(223); plot([1:nac],G(plot_on,:)); ylabel('Time [msec]'); grid;
    xlabel([int2str(nac),'th Order Log Area Ratio Coefficients']);
    ax=axis; ax(1)=1; ax(2)=nac; axis(ax);
    subplot(224); plot([1:nlpcc],LPCC(plot_on,:)); grid;
    xlabel([int2str(nlpcc),'th LPC Cepstral Coefficients']); ylabel('Time [msec]'); 
    ax=axis; ax(1)=1; ax(2)=nlpcc; axis(ax);
end

if strcmp(plot_on,'3D') | (isnumeric(plot_on) & plot_on~=0)
    plot3D(AC, 'surfi1', 'bone', t, [], -15, 65, ...
           [int2str(nac),'th Order Autocorrelation'], 'Time [msec]','',3);
    plot3D(A, 'surfi2', 'default', t, [], -15, 65, ...
           [int2str(nac),'th Order LPC Coefficients'], 'Time [msec]','',4);
    plot3D(K', 'surfi2', 'white', t, [], -15, 65, ...
           [int2str(nac),'th Order PARCOR Coefficients'], 'Time [msec]','',5);
    plot3D(G, 'surfi', 'pink', t, [], -15, 65, ...
           [int2str(nac),'th Order Log Area Ratio Coefficients'], 'Time [msec]','',6);
    plot3D(LPCC, 'meshi2', '', t, [], -13, 15, ...
           [int2str(nlpcc),'th Order LPC Cepstral Coefficients'], 'Time [msec]','',7);
end
