function [E] = extract_mel_energy(s, fe, win, pas, nmel, plot_on)
% s, fe - input signal and his sampling frequency
% win - dimensions in samples of the analysis window 
% pas - the distance/step between two consecutive windows
% nmel - number of mel bands (implicit 40)
% plot_on = 0 - disable the plot of graphics (implicit=1 enable)
%--------------------------------------------------------------------------
% Copyright(c) Zbancioc Marius-Dan, 2014, zmarius@etti.tuiasi.ro
% 1)Institute of Computer Science, Roumanian Academy, Iasi Branch, 
% 2)Technical University "Gheorghe Asachi" of Iasi, Romania
%--------------------------------------------------------------------------
% References:
%- M. Zbancioc, M. Feraru, The Analysis of the FCM and WKNN Algorithms Performance for the Emotional Corpus SROL, Advances in Electrical and Computer Engineering, Vol. 12, Issue 3, 2012, pp. 33  38, ISSN: 1582-7445, e-ISSN: 1844-7600, Digital Object Identifier: 10.4316/AECE.2012.03005
%- M. Feraru, M. Zbancioc, Speech Emotion Recognition for SROL Database using Weighted KNN algorithm, International Conference on Electronics, Computers and Artificial Intelligence (ECAI2013), June 27-29, 2013, Pitesti, Romania, ISBN: 978-1-4673-4935-2, Digital Object Identifier: 10.1109/ECAI.2013.6636189 2013 IEEE
%- M. Zbancioc, M. Feraru, A Study about MFCC Relevance in Emotion Classification for SROL Database, Proceedings of 4th International Symposium on Electrical and Electronics Engineering, ISEEE 2013, October, 11-13, 2013, Gala?i, Romnia, ISBN 978-1-4799-2441-7, IEEE Catalog Number CFP1393K-USB.
%- M. Zbancioc, M. Feraru, Using the Lyapunov Exponent from Cepstral Coefficients for Automatic Emotion Recognition, Proc of 8th IEEE International Conference on Electrical and Power Engineering  EPE 2014, 16-17 Oct., Ia?i, Romnia, ISSN: 978-1-4799-5848-1.
%- M. Feraru, M. Zbancioc, Emotion Recognition in Romanian Language using LPC Features, The 4th IEEE International Conference on E-Health and Bioengineering - EHB 2013, November 21-23, 2013, Ia?i, Romania, ISBN: 978-1-4799-2373-1/13/$31.00 2013 IEEE

dir_cur=pwd;
if nargin<2,
    [fname,fpath]=uigetfile('*.wav','Open');
    %if ~strcmp(dir_cur,fpath(1:end-1)), cd(fpath); end
    [s,fe]=wavread([fpath fname]);
end
if nargin<5, nmel=12; end
if nargin<6, plot_on=0; end

% Pre-emphasised filter with the transfer function H(z)=1-0.97z^-1
s=filter([1 -0.97],[1],s);

% The analysis windows have 25.6ms (409.6 samples for a sampling freq. fe=16KHz)
% and a step of 10 ms (overlapping 15.6/25.6). It was computed the power spectrum
% using Discrete Fourier Transform DFT in 512 points for fe=16kHz (if is necessary 
% the window will be compleated with zeros).
% If the sampling frequency (fe) has other value than the implicit one,
% will chose the nearest power of 2 for the analysis window dimension
if nargin<3, win = round(25.6/1000*fe); end
if nargin<4, pas = round(10/1000*fe); end
nfft = 2^(fix(log2(win))+1);

len = length(s);
iter=ceil((len-win)/pas);
t = (1:pas:len-win+1)*win/fe; nwin=iter; %nwin=length(t);
f = [0:nfft/2]*fe/nfft;	% positiv frequencies from the spectral window compleated with zeros

% % Computes the intervals of 40 mel-frequency bands:
% % 13 bands are liniar distributed in the interval 100-1000Hz and the other 
% % 27 band are logarithmic disptributed in the interval 1-7kHz 
% fmel = round([100:(1000-100)/13:1000]); %fmel=mel([100:1000],cceps,'Fant');
% fmel=[fmel(1:length(fmel)-1),mel([1000:fe/2],nmel-cceps,'Fant')];

%[W, fmel] = rectangular_mel_filter(nmel, nfft, fe);
[W, fmel] = triangular_mel_filter(nmel, nfft, fe);

%[W1, c, f] = triangular_mel_filter(nmel, 2^16, fe); %graphic of triangular filters 
%figure(1); subplot(311); plot(f,W1,'k'); xlabel( 'Frequency (Hz)' ); ylabel( 'Weight' ); axis tight


A = zeros(nwin,nfft/2+1);
E = zeros(nwin,nmel);

contor=1;
for i = 1 : pas : iter*pas
    Am=abs(fft(s(i:i+win-1).*hamming(win),nfft)); 
    % power spectrum = y*.conj(Y)/N=abs(Y).^2/N, where Y=fft(s,N)
    Am=Am(1:nfft/2+1).^2/nfft;
%     for j = 1 : nmel
%         E(contor,j)=sum(Am(f>=fmel(j) & f<fmel(j+1)));
%     end
    E(contor,:) = (W*Am)';
    A(contor,:)=Am'; 
    contor=contor+1;
end
%---------------------------------------------------------------------------------------------
function [W, fmel, f] = triangular_mel_filter(nf, nfft, fs, f_start, f_stop)
% nf - number of triangular filters
% nfft - number of points used by FFT transform
%       gives the dimension of vectors for each filter
% fs - samplig frequency
% Return a matrix with the weights values W of filters, respectivelly 
% the limits of the intervals covered by each filter (fmel)
if nargin<1, nf=40; end
if nargin<2, nfft=512; end
if nargin<3, fs=8000; end
if nargin<4, f_start=0; end
if nargin<5, f_stop=fs/2; end

len = nfft/2+1;
f = linspace( f_start, f_stop, len );

%fmel= mel([f_start:f_stop],nf,'Koening'); %'Fant')];
fmel = mel2hz( hz2mel(f_start)+[0:nf+1]*((hz2mel(f_stop)-hz2mel(f_start))/(nf+1)) );

W = zeros( nf, len);
for i = 1:nf 
    k = f>=fmel(i)&f<=fmel(i+1);          % panta crescatoare
    W(i,k) = (f(k)-fmel(i)) / (fmel(i+1)-fmel(i));
    k = f>=fmel(i+1)&f<=fmel(i+2);        % panta descrescatoare
    W(i,k) = (fmel(i+2)-f(k)) / (fmel(i+2)-fmel(i+1));
%     % normalization according with the area of each triangle (and the distance between two consecutive frequencies)
%     W(i,:) = 2*W(i,:)/((fmel(i+2)-fmel(i))*length(f)/(fs/2)); 
    W(i,:) = W(i,:) / sum(W(i,:));          % Impose the condition that the weights sum on each rows to be equal with 1
end

%---------------------------------------------------------------------------------------------
function [W, fmel, f] = rectangular_mel_filter(nf, nfft, fs, f_start, f_stop)
% nf - number of recangular filters
% nfft - number of points used by FFT transform
%       gives the dimension of vectors for each filter
% fs - sampling frequency
% Return a matrix with the weights values W of filters, respectivelly 
% the limits of the intervals covered by each filter (fmel)
if nargin<1, nf=40; end
if nargin<2, nfft=512; end
if nargin<3, fs=8000; end
if nargin<4, f_start=0; end
if nargin<5, f_stop=fs/2; end

len = nfft/2+1;
f = linspace( f_start, f_stop, len );

%fmel = mel([f_start:f_stop],nf,'Koening'); %'Fant')];
fmel = mel2hz( hz2mel(f_start)+[0:nf]*((hz2mel(f_stop)-hz2mel(f_start))/nf) );

W = zeros( nf, len);
for i = 1:nf 
    W(i,:) = f>=fmel(i)&f<fmel(i+1);
end

%---------------------------------------------------------------------------------------------
function mel = hz2mel(hz)
mel = 1127*log(1+hz/700); 

function hz = mel2hz(mel)
hz = 700*exp(mel/1127)-700;